Type: Package
Package: rsconnect
Title: Deploy Docs, Apps, and APIs to 'Posit Connect', 'shinyapps.io',
        and 'RPubs'
Version: 1.10.0
Authors@R: c(
    person("Aron", "Atkins", , "aron@posit.co", role = c("aut", "cre")),
    person("Toph", "Allen", role = "aut"),
    person("Hadley", "Wickham", role = "aut"),
    person("Jonathan", "McPherson", role = "aut"),
    person("JJ", "Allaire", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Programmatic deployment interface for 'RPubs',
    'shinyapps.io', and 'Posit Connect'. Supported content types include R
    Markdown documents, Shiny applications, Plumber APIs, plots, and
    static web content.
License: GPL-2
URL: https://rstudio.github.io/rsconnect/,
        https://github.com/rstudio/rsconnect
BugReports: https://github.com/rstudio/rsconnect/issues
Depends: R (>= 3.5.0)
Imports: cli, curl (>= 7.0.0), digest, httr2, jsonlite, lifecycle,
        openssl (>= 2.0.0), PKI, packrat (>= 0.6), renv (>= 1.0.0),
        rlang (>= 1.0.0), rstudioapi (>= 0.18.0), snowflakeauth, tools,
        yaml (>= 2.1.5), utils
Suggests: Biobase, BiocManager, foreign, knitr, MASS, plumber (>=
        0.3.2), quarto, reticulate, rmarkdown (>= 1.1), shiny, testthat
        (>= 3.1.9), webfakes, withr
VignetteBuilder: knitr, rmarkdown
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/Python/Note: Python <= 3.10 requires tomllib package for
        pyproject.toml parsing
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-08 19:16:41 UTC; aron
Author: Aron Atkins [aut, cre],
  Toph Allen [aut],
  Hadley Wickham [aut],
  Jonathan McPherson [aut],
  JJ Allaire [aut],
  Posit Software, PBC [cph, fnd]
Maintainer: Aron Atkins <aron@posit.co>
Repository: CRAN
Date/Publication: 2026-06-08 19:50:02 UTC
Built: R 4.5.2; ; 2026-06-08 22:05:43 UTC; unix
