% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rsconnect-package.R
\name{rsconnectPackages}
\alias{rsconnectPackages}
\title{Using Packages with rsconnect}
\description{
See ?\code{\link[=appDependencies]{appDependencies()}}
}
\keyword{internal}
